from nacl.encoding import HexEncoder
from nacl.public import Box, PrivateKey, PublicKey
from pydantic import SecretStr

from monstermash.datamodels import KeyPair


class Crypt:
    """
    Crypt class provides methods for encryption and decryption of messages using NaCl (Salt) cryptographic library.

    Attributes:
        key_length (int): The length of the keys used in encryption/decryption. Default is 32.
    """

    key_length: int = 32

    def __init__(self, private_key: bytes, message_encoder=HexEncoder, key_encoder=HexEncoder):
        """
        Initialize a new instance of Crypt class.

        Args:
            private_key (bytes): The private key used for encryption and decryption.
            message_encoder (HexEncoder): Encoder used for encoding and decoding messages. Default is HexEncoder.
            key_encoder (HexEncoder): Encoder used for encoding and decoding keys. Default is HexEncoder.
        """
        self.message_encoder = message_encoder
        self.key_encoder = key_encoder
        self._private_key = PrivateKey(private_key, encoder=self.key_encoder)

    @property
    def private_key(self) -> bytes:
        """Get encoded private key.

        Returns:
            bytes: The encoded private key.
        """
        return self._private_key.encode(self.key_encoder)

    @property
    def public_key(self) -> bytes:
        """Get raw public key.

        Returns:
            bytes: The raw public key.
        """
        return self._private_key.public_key._public_key

    @property
    def encoded_public_key(self) -> bytes:
        """Get encoded public key.

        Returns:
            bytes: The encoded public key.
        """
        return self.key_encoder.encode(self.public_key)

    @classmethod
    def generate(cls, encoder=HexEncoder) -> KeyPair:
        """
        Generate a new private-public key pair.

        Args:
            encoder (HexEncoder): Encoder used for encoding the keys. Default is HexEncoder.

        Returns:
            KeyPair: A tuple containing the encoded private key and public key.
        """
        private_key = PrivateKey.generate()
        private_key_encoded = private_key.encode(encoder)
        public_key_encoded = private_key.public_key.encode(encoder)
        return KeyPair(private_key=SecretStr(private_key_encoded.decode()), public_key=public_key_encoded)

    def encrypt(self, message: bytes, public_key: bytes) -> bytes:
        """
        Encrypt a message using a given public key. We append the public key to the ciphertext so that decryption
        becomes easy. The size of the key has to be consistent between encryption and decryption so that we can split
        the ciphertext based off the key size.

        Args:
            message (bytes): The message to be encrypted.
            public_key (bytes): The public key used for encryption.

        Returns:
            bytes: The encrypted message.
        """
        secret = Box(self._private_key, PublicKey(public_key, encoder=self.key_encoder))
        encrypted = secret.encrypt(message)
        formatted = b''.join([self.public_key, encrypted])
        return self.message_encoder.encode(formatted)

    def decrypt(self, message: bytes) -> bytes:
        """
        Decrypt a message using the private key.

        Args:
            message (bytes): The message to be decrypted.

        Returns:
            bytes: The decrypted message.
        """
        decoded = self.message_encoder.decode(message)
        key = self.key_encoder.encode(decoded[: self.key_length])
        data = decoded[self.key_length :]
        secret = Box(self._private_key, PublicKey(key, encoder=self.key_encoder))
        return secret.decrypt(data)



from configparser import ConfigParser


class ConfigManager:
    """ConfigManager

    ConfigManager class manages the reading and writing operations to a configuration file. This is useful for
    generating a set of keys (using `monstermash generate`) that will be used for future encryption and decryption
    operations
    """

    def __init__(self, config_file: str):
        """
        Initialize a new instance of ConfigManager class.

        Args:
            config_file (str): The path of the configuration file. Will default to ~/.monstermashcfg
        """
        self.parser = ConfigParser()
        self.config_file = config_file

    def read(self) -> ConfigParser:
        """
        Read the configuration file and returns a ConfigParser object.

        Returns:
            ConfigParser: The ConfigParser object with the contents of the configuration file.
        """
        self.parser.read(self.config_file)
        return self.parser

    def write(self, config: ConfigParser):
        """
        Write a ConfigParser object to the configuration file.

        Args:
            config (ConfigParser): The ConfigParser object to be written to the file.
        """
        with open(self.config_file, 'w+') as f:
            config.write(f)



from pydantic import BaseModel, SecretStr


class KeyPair(BaseModel):
    """
    KeyPair class representing a pair of private and public keys.

    Attributes:
        private_key (SecretStr): The private key used in the encryption or decryption process.
        public_key (str): The public key used in the encryption or decryption process.
    """

    private_key: SecretStr
    public_key: str



import os

from pydantic import field_validator
from pydantic_settings import BaseSettings


class Config(BaseSettings):
    """
    Config class provides a model for application configuration settings,
    which includes the path to the configuration file.

    Attributes:
        config_file (str): The path to the configuration file. Default is '~/.monstermashcfg'.
    """

    config_file: str = '~/.monstermashcfg'

    @field_validator('config_file')
    def validate_file(cls, v: str) -> str:
        """
        Validate and normalize the path of the configuration file.

        This validator check that if the config_file path starts with '~', it is expanded to the full path in the
        user's home directory. If not we store it where you have asked.

        Args:
            v (str): The original path string.

        Returns:
            str: The normalized file path.
        """
        if v.startswith('~'):
            home = os.path.expanduser('~')
            return os.path.join(home, v.replace('~/', ''))
        return v



from typing import Mapping, Optional

import click

from monstermash.config import Config
from monstermash.crypt import Crypt
from monstermash.parser import ConfigManager
from monstermash.utils.file import NEW_LINE_EXPR, open_file


@click.group()
@click.pass_context
def mash(ctx):
    config = Config()
    ctx.obj = ConfigManager(config.config_file)


@mash.command()
def generate():
    keys = Crypt.generate()

    click.echo('-----------------')
    click.echo('Private Key (keep it secret, keep it safe)')
    click.echo(keys.private_key.get_secret_value())
    click.echo('Public Key (you can share this one)')
    click.echo(keys.public_key)
    click.echo('-----------------')


@mash.command()
@click.pass_obj
@click.option(
    '--profile',
    prompt='Profile name',
    default='default',
    help='The name of the profile you want to create.',
)
@click.option('--private-key', prompt='Your private key', help='Your private key.')
@click.option('--public-key', prompt='Your public key', help='Your public key.')
def configure(config_manager: ConfigManager, profile: str, private_key: str, public_key: str):
    config = config_manager.read()
    config[profile] = {'private_key': private_key, 'public_key': public_key}
    config_manager.write(config)
    click.echo('.monstermashcfg file in your root directory contains the config')


@mash.command()
@click.pass_obj
@click.option('--profile', default=None)
@click.option('--private-key', default=None, help='Your private key.')
@click.option('--public-key', default=None, help='Your public key.')
@click.option('--file', default=None, help='The path to the file you want to encrypt')
@click.option('--data', default=None, help='Input data you want to encrypt')
def encrypt(
    config_manager: ConfigManager,
    profile: Optional[str],
    private_key: Optional[str],
    public_key: Optional[str],
    file: Optional[str],
    data: Optional[str],
):
    if profile is not None:
        config = config_manager.read()
        private_key = config[profile]['private_key']
        public_key = config[profile]['public_key'] if public_key is None else public_key

    if private_key is None:
        raise ValueError('you must specify either a private key or profile')

    crypt = Crypt(private_key.encode())

    if file is not None:
        data = open_file(file)

    if data is None:
        raise ValueError('you must specify either --file or --data')

    if public_key is None:
        raise ValueError('a public key is required for encryption')

    encrypted = crypt.encrypt(data.encode(), public_key.encode())
    click.echo(encrypted)


@mash.command()
@click.pass_obj
@click.option('--profile', default=None)
@click.option('--private-key', default=None, help='Your private key.')
@click.option('--file', default=None, help='The path to the file you want to encrypt')
@click.option('--data', default=None, help='Input data you want to encrypt')
def decrypt(
    config_manager: ConfigManager,
    profile: Optional[str],
    private_key: Optional[str],
    file: Optional[str],
    data: Optional[str],
):
    if profile is not None:
        config = config_manager.read()
        private_key = config[profile]['private_key']

    if private_key is None:
        raise ValueError('you must specify either a private key or profile')

    crypt = Crypt(private_key.encode())

    if file is not None:
        data = open_file(file)

    if data is None:
        raise ValueError('you must specify either --file or --data')

    decrypted = crypt.decrypt(NEW_LINE_EXPR.sub('', data).encode())
    click.echo(decrypted)


def _echo_config(section: str, config: Mapping):
    click.echo(f'[{section}]')
    click.echo(f"private_key: {config[section]['private_key']}")
    click.echo(f"public_key: {config[section]['public_key']}")
    click.echo('----------------')


@mash.command()
@click.option('--profile', default=None)
@click.pass_obj
def getconfig(config_manager: ConfigManager, profile: str):
    config = config_manager.read()
    if profile:
        _echo_config(profile, config)
    else:
        for section in config.sections():
            _echo_config(section, config)


if __name__ == '__main__':
    mash()



import json
import re
from typing import Union

from pydantic import Json

NEW_LINE_EXPR = re.compile(r'[\n\r]')


def open_file(file) -> Union[Json, str]:
    """
    Open a file and return its contents. For JSON files, the contents are returned as a dictionary.
    For non-JSON files, the contents are returned as a string.

    Args:
        file (str): The path to the file to open.

    Returns:
        Union[dict, str]: The contents of the file. If the file is a JSON file, a dictionary is returned. Otherwise, a
            string is returned.
    """
    if re.search('.json$', file):
        with open(file, 'r') as f:
            data = json.load(f)
    else:
        with open(file, 'r') as f:
            data = f.read()
    return data
